// -*-mode:c++; c-style:k&r; c-basic-offset:4;-*-
//
// Copyright 2016, Julian Catchen <jcatchen@illinois.edu>
//
// This file is part of Chromonomer.
//
// Chromonomer is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Chromonomer is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Chromonomer.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef __MARKER_H__
#define __MARKER_H__

#include <utility>
using std::pair;
using std::make_pair;
#include <string>
using std::string;
#include <vector>
using std::vector;

#include "constants.h"
#include "Seq.h"

class Marker {
public:
    uint        id;
    //
    // Alignment properties.
    //
    string      scaffold;
    uint        bp;
    strand_type strand;
    string      cigar;
    uint        aln_cnt;

    //
    // Genetic map properties.
    //
    string      lg;
    double      cM;

    //
    // Record previous scaffold/basepairs if marker was relocated.
    //
    vector<pair<string, int> > old;

    Marker() {
	this->id = 0;
	this->bp = 0;
	this->cM = 0.0;
        this->aln_cnt = 0;
    }
    Marker(uint id) {
	this->id = id;
	this->bp = 0;
	this->cM = 0.0;
        this->aln_cnt = 0;
    }
    Marker(uint id, string lg, double cM) {
	this->id = id;
	this->lg = lg;
	this->cM = cM;
	this->bp = 0;
        this->aln_cnt = 0;
    }
};

#endif // __MARKER_H__
